function [numblobs, density, blobchars]=blobcomps(mat,voisins)

% Function to identify blobs in a binary matrix and calculate their characteristics.  
% A blob is defined as a contiguous region of cells in the 3D space.

% INPUTS:
% mat - a 3D matrix containing zeros and ones, where a one indicates presence of the blob type of interest
% voisins - the neighbourhood size (in 3D, typically 6, 18, or 26) Use 6 for a von Neumann neighbourhood and use 26 for a Moore
% neighbourhood. 

% See Matlab documentation on bwlabeln for more info.  

% OUTPUTS:
% numblobs - number of unique blobs in the matrix
% density - density of blobs in the matrix
% blobchars - a 2D array of of size [numblobs,3] where:

% column 1 gives the blob volume,
% column 2 gives the blob bounding box volume and 
% column 3 gives the blob shape complexity, defined as the ratio of the blob volume to the
% bounding box volume. Bounding box volume should always be equal to or greater than blob volume
% so the max. value of the ratio is 1 and the value tends to zero for more
% complex shapes.

% Written by Lael Parrott 2008 & 2007

% Distributed as part of the 3D Metrics Toolbox

% Licensed under a Creative Commons Attribution-NonCommercial-Share Alike 3.0 Unported License.




% Use internal bwlabel function to locate blobs
[L,numblobs]=bwlabeln(mat,voisins);

% Print number of blobs on screen
numblobs

% Calculate the density of this blob type
density = nnz(mat)/prod(size(mat))

% Allocate memory for blobchars matrix
blobchars=zeros(numblobs,3);

% Calculate blob properties

stats=regionprops(L,'Area','Boundingbox');
allarea=[stats.Area];
allarea=allarea';
bbox=[stats.BoundingBox];
bbox2=reshape(bbox,6,numblobs)';
bbox2=bbox2(:,4:6);
bboxvol=bbox2(:,1).*bbox2(:,2).*bbox2(:,3);


ratio=allarea./bboxvol;
volumes = allarea;

blobchars(:,1) = volumes;
blobchars(:,2) = bboxvol;
blobchars(:,3) = ratio;

% Print out some statistics

maxblobvolume=max(volumes)

maxblob=find(allarea==(max(allarea)));

meanblobcomplexity=mean(ratio)
stdblobcomplexity=std(ratio)
complexityoflargestblob=ratio(maxblob)

% Generate figure showing frequency distribution of blob volumes
% Note that a custom figure can be made for your data using the values in
% blobchars

figure()

hist(log(volumes),20)
xlabel('Blob volume (logarithmic scale)')
ylabel('Frequency')
title('Frequency distribution of blob volumes')

% End

return 

